@@ -20,15 +20,16 @@ firm.eff <- rnorm(nlevels(firm))
 y <- x + 0.25*x2 + 0.5*x3 + 0.17*x4 + 0.8*x5 -0.3*x6 + id.eff[id] + firm.eff[firm] + rnorm(length(x))
 
 ## estimate
-summary(est <- felm(y ~ x+x2 + x3 + x4 + x5 + x6 + G(id) + G(firm)))
+est <- felm(y ~ x+x2 + x3 + x4 + x5 + x6 + G(id) + G(firm))
 ## extract the group fixed effects
 alpha <- getfe(est)
-alpha
-summary(lm(y ~ x + x2 + x3 + x4 + x5 + x6 + id + firm-1))
+#summary(lm(y ~ x + x2 + x3 + x4 + x5 + x6 + id + firm-1))
 
 # merge back
 ideff <- alpha[paste('id',id,sep='.'),'effect']
 firmeff <- alpha[paste('firm',firm,sep='.'),'effect']
 
 ## verify that id and firm coefficients are 1
-summary(lm(y ~ x + x2 + x3 + x4 + x5 + x6 + ideff + firmeff-1),digits=8)
+cat('accuracy:',coef(lm(y ~ x + x2 + x3 + x4 + x5 + x6 + ideff + firmeff-1))[7:8]-1,'\n')
+
+
