@@ -1,5 +1,5 @@
 library(lfe)
-options(lfe.threads=4)
+options(lfe.threads=2)
 set.seed(42)
 x <- rnorm(1000)
 x2 <- rnorm(length(x))
@@ -13,11 +13,12 @@ firm.eff <- rnorm(nlevels(firm))
 ## left hand side
 u <- rnorm(length(x))
 x3 <- rnorm(length(x))
-Q <- 0.3*x3 + x + 0.2*x2 + id.eff[id] + 0.7*u + rnorm(length(x),sd=0.3)
-
-y <- x + 0.5*x2 + id.eff[id] + firm.eff[firm] + Q + u
+x4 <- rnorm(length(x))
+Q <- 0.3*x3 + x + 0.2*x2 + id.eff[id] + 0.15*u + rnorm(length(x),sd=0.2)
+R <- 0.5*x4 + 0.2*x + 0.5*x2 + 0.7*id.eff[id] - 0.11*u + rnorm(length(x),sd=0.2)
+y <- x + 0.5*x2 + id.eff[id] + firm.eff[firm] + Q + R + u
 
 ## estimate and print result
-est <- felm(y ~ x+x2+G(id)+G(firm)+Q,iv=Q~x3)
+est <- felm(y ~ x+x2+G(id)+G(firm)+Q+R,iv=list(Q ~ x3, R ~ x4))
 summary(est,robust=TRUE)
 
